package io.scanbot.sdk.flutter.camera.detectorWrappers

import android.content.pm.PackageManager
import android.graphics.Color
import android.util.Log
import com.google.gson.Gson
import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodChannel
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.camera.CaptureInfo
import io.scanbot.sdk.camera.FrameHandlerResult
import io.scanbot.sdk.contourdetector.ContourDetectorFrameHandler
import io.scanbot.sdk.docdetection.ui.DocumentScannerView
import io.scanbot.sdk.docdetection.ui.IDocumentScannerViewCallback
import io.scanbot.sdk.flutter.*
import io.scanbot.sdk.flutter.camera.CustomViewDisplayOwner
import io.scanbot.sdk.flutter.camera.checkParam
import io.scanbot.sdk.flutter.camera.config.DocumentScannerConfiguration
import io.scanbot.sdk.flutter.utils.convertDpToPx
import io.scanbot.sdk.persistence.Page
import io.scanbot.sdk.persistence.PageImageSource
import io.scanbot.sdk.ui.camera.CameraUiSettings
import io.scanbot.sdk.ui.camera.ScanbotCameraContainerView
import io.scanbot.sdk.ui.camera.ScanbotCameraXView
import kotlinx.coroutines.*


class DocumentDetector(
    private val scannerView: DocumentScannerView,
    private val coroutineScope: CoroutineScope,
    private val cameraChannel: MethodChannel,
    private val detectorChannel: MethodChannel,
    override val scanbotSDK: ScanbotSDK,
    private val customViewDisplayOwner: CustomViewDisplayOwner,
    private val creationParams: HashMap<String, Any>,
    val gson: Gson
) : IDetectorWrapper, CoroutineScope by coroutineScope {
    private val returnContext = Dispatchers.Main
    private val pageProcessor = scanbotSDK.createPageProcessor()
    private val pageFileStorage = scanbotSDK.createPageFileStorage()
    private val contourDetector = scanbotSDK.createContourDetector()
    private var isSavingSnapshot = false
    var documentScannerConfig: DocumentScannerConfiguration = DocumentScannerConfiguration()
    private val handler = ContourDetectorFrameHandler.ResultHandler { result ->
        Log.d("CAMERA", "FRAME RESULT RETURNED")
        runBlocking {
            when (result) {
                is FrameHandlerResult.Success -> {
                    result.value.let { result ->
                        try {
                            val json = gson.toJsonTree(result).asJsonObject
                            withContext(returnContext) {
                                detectorChannel.invokeMethod(
                                    "onContourDetected",
                                    gson.toJson(json)
                                )
                            }
                        } catch (e: Throwable) {
                            e.printStackTrace()
                        }
                    }
                }
                is FrameHandlerResult.Failure -> {
                    withContext(returnContext) {
                        val json = scanbotSDK.getLicenseStatusJson()
                        detectorChannel.invokeMethod(
                            "onLicenseError",
                            json.toString()
                        )
                    }
                }
            }
        }
        false
    }

    override fun attach(arguments: Json) {
        creationParams.apply {
            scannerView.initCamera(CameraUiSettings(true))
            val cameraXView =
                scannerView.findViewById<ScanbotCameraContainerView>(R.id.scanbot_camera_view)
            (cameraXView.getChildAt(0) as ScanbotCameraXView).lifecycleOwner =
                customViewDisplayOwner

            checkContourConfig()

            val configuration =
                (arguments).toDocumentScannerConfig(gson)
            documentScannerConfig?.detectDocumentAfterSnap =
                configuration.detectDocumentAfterSnap as Boolean
            addDetectorMethodHandler()
            scannerView.initDetectionBehavior(
                contourDetector,
                handler,
                object : IDocumentScannerViewCallback {
                    override fun onCameraOpen() {
                        creationParams.apply {
                            launch {
                                withContext(returnContext) {

                                    checkParam<Boolean>("flashEnabled")?.let {
                                        scannerView.viewController.useFlash(it)
                                    }
                                    checkParam<Double>("cameraZoomFactor")?.let {
                                        cameraXView.setOpticalZoomLevel(it.toFloat())
                                    }
                                    cameraChannel.invokeMethod("onCameraOpen", "")
                                }
                            }
                        }
                    }

                    override fun onPictureTaken(image: ByteArray, captureInfo: CaptureInfo) {
                        //todo possibly need to fix threading--

                        launch {
                            saveSnappedImageAndReturn(image, captureInfo)
                            withContext(Dispatchers.Main) { scannerView.viewController.startPreview() }
                            isSavingSnapshot = false
                        }
                    }
                })
            scannerView.viewController.isFrameProcessingEnabled = true
            checkScannerConfig(configuration)
            scannerView.viewController.autoSnappingEnabled = true
        }
    }

    private fun HashMap<String, Any>.checkContourConfig() {
        checkParam<Json>("contourConfiguration") {
            it.toContourConfig(gson).let {
                scannerView.polygonConfiguration.setPolygonViewVisible(isVisible = true)

                it.autoSnapProgressEnabled?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnapProgressEnabled(it)
                }
                it.fillColor?.let {
                    scannerView.polygonConfiguration.setPolygonFillColor(Color.parseColor(it))
                }
                it.fillColor?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeColor(
                        Color.parseColor(
                            it
                        )
                    )
                }
                it.fillOkColor?.let {
                    scannerView.polygonConfiguration.setPolygonFillColorOK(Color.parseColor(it))
                }
                it.strokeColor?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeColor(Color.parseColor(it))
                }
                it.strokeOkColor?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeColorOK(Color.parseColor(it))
                }
                it.autoSnapProgressStrokeColor?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeColor(
                        Color.parseColor(
                            it
                        )
                    )
                }
                it.cornerRadius?.let {
                    scannerView.polygonConfiguration.setPolygonCornerRadius(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
                it.autoSnapProgressStrokeWidth?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeWidth(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
                it.strokeWidth?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeWidth(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
            }
        }
    }

    private fun checkScannerConfig(configuration: DocumentScannerConfiguration) {
        configuration.acceptedAngleScore?.let {
            scannerView.viewController.setAcceptedAngleScore(it.toDouble())
        }
        configuration.acceptedSizeScore?.let {
            scannerView.viewController.setAcceptedSizeScore(it.toDouble())
        }
        configuration.autoSnapSensitivity?.let {
            scannerView.viewController.setAutoSnappingSensitivity(it)
        }
        configuration.requiredAspectRatios?.let {
            scannerView.viewController.setRequiredAspectRatios(it)
        }
        configuration.ignoreBadAspectRatio?.let {
            scannerView.viewController.setIgnoreBadAspectRatio(it)
        }
        configuration.autoSnapEnabled?.let {
            scannerView.viewController.autoSnappingEnabled = it
        }
    }

    private suspend fun saveSnappedImageAndReturn(image: ByteArray, captureInfo: CaptureInfo) {
        try {
            withContext(returnContext) { cameraChannel.invokeMethod("onShowHeavyProcessing", true) }

            val pageImageSource =
                if (captureInfo.isCapturedAutomatically) PageImageSource.AUTOSNAPPING else PageImageSource.MANUAL_SNAP

            ///TODO apply  PageStorageProcessor.Configuration with image source and orientation of the image from version 1.95
            val pageId = pageFileStorage.add(image, pageImageSource)
            var page = Page(pageId, pageImageSource = pageImageSource)
            documentScannerConfig?.detectDocumentAfterSnap?.takeIf { it }?.let {
                page = pageProcessor.detectDocument(page)
            }

            val pageJson = page.toJson(gson)
            insertPageUris(scanbotSDK, coroutineScope, pageId, pageJson as JsonObject)
            withContext(returnContext) {
                cameraChannel.invokeMethod(
                    "onShowHeavyProcessing",
                    false
                )
            }
            withContext(returnContext) {
                cameraChannel.invokeMethod(
                    "onPictureTaken",
                    pageJson.toString()
                )
            }
        } catch (e: Throwable) {
            Log.e("ScanbotCameraModule", "error while saving snapped image", e)
        }
    }


    private fun addDetectorMethodHandler() {
        detectorChannel.setMethodCallHandler { call, result ->
            when (call.method) {
                "disable" -> {
                    scannerView.viewController.isFrameProcessingEnabled = false
                    result.success("success")
                }
                "enable" -> {
                    scannerView.viewController.isFrameProcessingEnabled = true
                    result.success("success")
                }
                "makeSnap" -> {
                    scannerView.viewController.takePicture(true)
                }
                "setAutoSnappingEnabled" -> {
                    val autoSnappingEnabled = call.argument<Boolean>("enabled") ?: false
                    scannerView.viewController.autoSnappingEnabled = autoSnappingEnabled
                    scannerView.polygonConfiguration.setPolygonViewVisible(isVisible = autoSnappingEnabled)
                    result.success("success")
                }
            }
        }
    }

    override fun detach() {
        detectorChannel.setMethodCallHandler(null)
    }

    override fun toggleContinuousFocus() {
        scannerView.viewController.continuousFocus()
    }

    override fun isFlashAvailable(): Boolean {
        return scannerView.context.packageManager.hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH)
    }

    override fun flashEnabled(enabled: Boolean) {
        scannerView.viewController.useFlash(enabled)
    }
}
