//
//  BarcodeScanner.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 27.06.22.
//

import Foundation
import ScanbotSDK
import UIKit

class BarcodeScanner: NSObject, SBSDKScanner, SBSDKBarcodeScannerViewControllerDelegate, CallErrorHandler  {
    
    private var viewController: SBSDKBarcodeScannerViewController?

    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var waitingForImageSnap: Bool = false
    private var configuration: [String:Any?]
    private var imageGenerationType: SBSDKBarcodeImageGenerationType = SBSDKBarcodeImageGenerationType.none
    private var cachedCodes: [SBSDKBarcodeScannerResult]? = nil
    var shouldRunDetection: Bool = true

    init(
    cameraMethodChannel: FlutterMethodChannel?,
    detectorChannel: FlutterMethodChannel?,
    finderChannel: FlutterMethodChannel?,
    configuration: [String:Any?]
    ){
        self.cameraMethodChannel = cameraMethodChannel
        self.detectorChannel = detectorChannel
        self.finderChannel = finderChannel
        self.configuration = configuration
    }
    
    func createNativeView(view _view: UIView){
        
            if(!ScanbotSDK.isLicenseValid()){
                DispatchQueue.main.async { [weak self] in
                    guard let callHandler = self?.detectorChannel else {
                        fatalError("callHandler cannot be nil")
                    }
                    self?.sendLicenseErrorToDetector(callHandler)
                }
                return
            }
        
            let rootViewController = UIViewController()
            rootViewController.view = _view
            viewController = SBSDKBarcodeScannerViewController(parentViewController: rootViewController,
                                                                     parentView: nil)
            viewController?.delegate = self
        
            setBarcodeScannerConfiguration(configuration:configuration)
            cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments:"")
    }
    
    func attach(view _view: UIView) {
        createNativeView(view: _view)
        self.detectorChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleDetectorCall(call: call, result: result)
        })
    }
    
    func setFlash(enabled: Bool) {
      viewController?.isFlashLightEnabled = enabled
    }
    
    func isFlashAvailable() -> Bool {
         return viewController?.isFlashLightAvailable ?? false
    }

    
    func detach(){
        detectorChannel?.setMethodCallHandler(nil)
        viewController?.dismiss(animated: true)
        viewController = nil
    }
    
    func barcodeScannerController(_ controller: SBSDKBarcodeScannerViewController, didCaptureBarcodeImage barcodeImage: UIImage) {
        // to snap picture
        
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: true)
        
        guard let uuid = SBSDKUIPageFileStorage.default().add(barcodeImage) else { return  }
        let uri = SBSDKUIPageFileStorage.default().imageURL(withPageFileID: uuid, pageFileType: .original)
        
        guard let codes = cachedCodes else { return }
        sendDetectedBarcodesResult(codes, uri?.standardizedFileURL.absoluteString ?? "")
        
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: false)
        waitingForImageSnap = false
    }
    
    func barcodeScannerControllerShouldDetectBarcodes(_ controller: SBSDKBarcodeScannerViewController) -> Bool {
        return shouldRunDetection && !waitingForImageSnap
    }
    
    func didChangeViewFinderRect(rect: CGRect) {
        let dictionary: [String: Any?] = ["left": rect.minX,
                                          "top": rect.minY,
                                          "right": rect.maxX,
                                          "bottom": rect.maxY,]
        
        finderChannel?.invokeMethod("finderSizeChanged", arguments: dictionary)
    }
    
    fileprivate func sendDetectedBarcodesResult(_ codes: [SBSDKBarcodeScannerResult],_ barcodeImageURI: String?) {
        let resultDict: [String: Any?] = ["barcodeItems": codes.map({ $0.encodedDictionary }), "barcodeImageURI" : barcodeImageURI]
        
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.detectorChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod("onBarcodeDetected", arguments: resultDict.encodedString)
        }
    }
    
    func barcodeScannerController(_ controller: SBSDKBarcodeScannerViewController, didDetectBarcodes codes: [SBSDKBarcodeScannerResult], on image: UIImage) {
        if(!codes.isEmpty){
            switch(imageGenerationType){
            case .none:
                sendDetectedBarcodesResult(codes, nil)
            case .fromVideoFrame:
                waitingForImageSnap = true
                cachedCodes = codes
            case .capturedImage:
                waitingForImageSnap = true
                cachedCodes = codes
            @unknown default:
                sendDetectedBarcodesResult(codes, nil)
            }
        }
    }
    
    
    func setBarcodeScannerConfiguration(configuration:[String:Any?]?) {
        
        viewController?.viewFinderRectDidChangeHandler = {(finderRect:CGRect) -> Void in
            self.didChangeViewFinderRect(rect:finderRect)
        }
        
        if let flashEnabled = configuration?["flashEnabled"] as? Bool {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = configuration?["cameraZoomFactor"] as? CGFloat {
            let zoomConfig = SBSDKBaseScannerZoomConfiguration.default()
            zoomConfig.initialZoomFactor = cameraZoomFactor
            viewController?.zoomConfiguration = zoomConfig
        }
        
        if let finderConfiguration: [String:Any] = configuration?["finder"] as? [String : Any] {
            let finderConfig = SBSDKBaseScannerViewFinderConfiguration.default()
            finderConfig.backgroundColor = UIColor(hexString: "#00000000")
            finderConfig.lineColor = UIColor(hexString: "#00000000")
            finderConfig.isViewFinderEnabled = true
            
            
            if
                let finderAspectRatio = finderConfiguration["finderAspectRatio"] as? [String:Any],
                let width = finderAspectRatio["width"] as? Double,
                let height = finderAspectRatio["height"] as? Double
            {
                finderConfig.aspectRatio = SBSDKAspectRatio(width: width, andHeight: height)
            }
            
            if let padding = finderConfiguration["finderMinimumPadding"] as? CGFloat {
                finderConfig.minimumInset.top = padding
                finderConfig.minimumInset.bottom = padding
                finderConfig.minimumInset.left = padding
                finderConfig.minimumInset.right = padding
            }
            viewController?.viewFinderConfiguration = finderConfig
        }
        if let scannerConfiguration: [String:Any] = configuration?["scannerConfiguration"] as? [String : Any] {
            
            if scannerConfiguration["acceptedDocumentFormats"] is [String] {
                viewController?.acceptedDocumentTypes = SBSDKBarcodeDocumentType.fromArguments(scannerConfiguration)
            }
            
            if scannerConfiguration["barcodeFormats"] is [String] {
                viewController?.acceptedBarcodeTypes = SBSDKBarcodeType.fromArguments(scannerConfiguration)
            }
            
            if scannerConfiguration["engineMode"] is String {
                viewController?.engineMode = SBSDKBarcodeEngineMode.fromArguments(scannerConfiguration)!
            }
            
            if let additionalParams: [String : Any] = scannerConfiguration["additionalParameters"] as? [String : Any] {
                viewController?.additionalDetectionParameters = SBSDKBarcodeAdditionalParameters(from:additionalParams)
            }
            
            if let value = scannerConfiguration["gs1DecodingEnabled"] as? Bool {
                viewController?.additionalDetectionParameters.enableGS1Decoding = value
            }
            
            if let value = scannerConfiguration["minimumTextLength"] as? UInt {
                viewController?.additionalDetectionParameters.minimumTextLength = value
            }
            
            if let value = scannerConfiguration["maximumTextLength"] as? UInt {
                viewController?.additionalDetectionParameters.maximumTextLength = value
            }
            
            if let value = scannerConfiguration["minimum1DBarcodesQuietZone"] as? UInt {
                viewController?.additionalDetectionParameters.minimum1DBarcodesQuietZone = value
            }
            
            if let value = scannerConfiguration["msiPlesseyChecksumAlgorithm"] as? String {
                viewController?.additionalDetectionParameters.msiPlesseyChecksumAlgorithm = SBSDKBarcodeMSIPlesseyChecksumAlgorithm(from: value)
            }
            
            if let value = scannerConfiguration["stripCheckDigits"] as? Bool {
                viewController?.additionalDetectionParameters.stripCheckDigits = value
            }
            
            if let barcodeImageGenerationType = scannerConfiguration["barcodeImageGenerationType"] as? String {
                imageGenerationType = SBSDKBarcodeImageGenerationType(from:barcodeImageGenerationType)
                viewController?.barcodeImageGenerationType = imageGenerationType
            }
        }
        
    }
    
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "enable" : shouldRunDetection = true
        case "disable" : shouldRunDetection = false
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result) //always return some result to the flutter
    }
    
    func resurnSuccess(result: @escaping FlutterResult){
        result("success")
    }
}
